@@ -91,7 +91,7 @@ my $hasFBO = 0;
 my $hasVBO = 0;
 my $hasFragProg = 0;
 my $hasImagePointer = 0;
-my $idleTime = time();
+my $idleTime = $hasHires ? gettimeofday() : time();
 my $idleSecsMax = 5;
 my $er;
 
@@ -791,11 +791,13 @@ sub ourInitShaders
 sub cbRenderScene
 {
   # Quit if inactive
-  if ($idleSecsMax < (time()-$idleTime))
+  my $time = $hasHires ? gettimeofday() : time();
+  my $time_to_exit = $idleSecsMax - ($time-$idleTime);
+  if ($time_to_exit <= 0 )
   {
     print "Idle timeout; completing test\n";
     ourCleanup();
-    exit(0);
+    return quit("render callback");
   }
 
   my $buf; # For our strings.
@@ -989,6 +991,10 @@ sub cbRenderScene
   # But, for fun, let's make the text partially transparent too.
   glColor4f(0.6,1.0,0.6,.75);
 
+  $buf = sprintf "TIME TO EXIT: %.1fs", $time_to_exit;
+  my $bufwidth = 6 * length $buf;
+  glRasterPos2i($Window_Width-4-$bufwidth,2); ourPrintString(GLUT_BITMAP_HELVETICA_12,$buf);
+
   # Render our various display mode settings.
   $buf = sprintf "Mode: %s", $TexModesStr[$Curr_TexMode];
   glRasterPos2i(2,2); ourPrintString(GLUT_BITMAP_HELVETICA_12,$buf);
@@ -1110,6 +1116,7 @@ sub Inset
   {
     my $frame = new OpenGL::Image(engine=>'Magick',
       width=>$Inset_Width, height=>$Inset_Height);
+    die $@ if $@;
     my($fmt,$size) = $frame->Get('gl_format','gl_type');
 
     glReadPixels_c( $Capture_X, $Capture_Y, $Inset_Width, $Inset_Height,
@@ -1241,7 +1248,7 @@ sub cbKeyPressed
   if ($key == 27 or $c eq 'Q')
   {
     ourCleanup();
-    exit(0);
+    return quit("key press callback");
   }
   elsif ($c eq 'B')
   {
@@ -1320,7 +1327,7 @@ sub cbKeyPressed
     printf "KP: No action for %d.\n", $key;
   }
 
-  $idleTime = time();
+  $idleTime = $hasHires ? gettimeofday() : time();
 }
 
 # ------
@@ -1359,7 +1366,7 @@ sub cbSpecialKeyPressed
     printf "SKP: No action for %d.\n", $key;
   }
 
-  $idleTime = time();
+  $idleTime = $hasHires ? gettimeofday() : time();
 }
 
 # ------
@@ -1465,7 +1472,7 @@ sub cbMouseClick
     print "\n";
   }
 
-  $idleTime = time();
+  $idleTime = $hasHires ? gettimeofday() : time();
 }
 
 sub GetKeyModifier
@@ -1496,7 +1503,7 @@ sub cbResizeScene
   $Window_Width  = $Width;
   $Window_Height = $Height;
 
-  $idleTime = time();
+  $idleTime = $hasHires ? gettimeofday() : time();
 }
 
 sub cbWindowStat
@@ -1512,6 +1519,31 @@ sub cbClose
   ReleaseResources();
 }
 
+# this is a little complicated
+# Using freeglut, doing a straight exit crashes on some systems, mostly observed
+# on windows, likely due to thread issues.
+# However on non-freeglut systems the proper way of using glutLeaveMainLoop is
+# not available.
+# So the proper one needs to be chosen.
+# However while exit exits the thread, glutLeaveMainLoop only sets a flag for
+# the event loop, thus we must take care to return when using it. Additionally
+# any use of quit() ALSO needs to return.
+#
+#   return quit();
+#
+sub quit {
+  my ($context) = @_;
+  $context ||= "<unknown context>";
+  print "Exiting in $context using ";
+  if (Acme::MITHALDU::BleedingOpenGL::_have_freeglut()) {
+    print "glutLeaveMainLoop (freeglut)\n";
+    glutLeaveMainLoop();
+    return;
+  }
+  print "perl exit(0)\n";
+  exit(0);
+}
+
 # ------
 # The main() function.  Inits OpenGL.  Calls our own init function,
 # then passes control onto OpenGL.
@@ -1640,12 +1672,14 @@ Press 'c' to capture/save a RGBA targa file.
 # Pass off control to OpenGL.
 # Above functions are called as appropriate.
 if (Acme::MITHALDU::BleedingOpenGL::_have_freeglut()) {
-   print "Setting window close to trigger return from mainloop.\n";
+   print "Setting window close to trigger return from mainloop (freeglut).\n";
    glutSetOption(GLUT_ACTION_ON_WINDOW_CLOSE,GLUT_ACTION_GLUTMAINLOOP_RETURNS)
 }
 
+print "Entering glutMainLoop\n";
 glutMainLoop();
+print "Returned from glutMainLoop\n";
 
-print "FreeGLUT returned from MainLoop\n";
+print "Exiting in main thread\n";
 
 __END__
